package morpion.modele.partie;

import morpion.modele.plateau.ContenuCase;
import java.util.ArrayList;
import java.util.List;
import morpion.modele.plateau.PlateauMorpionModele;

/**
 * Un modele pour le morpion. La taille du plateau est pour l'instant fixée à
 * 3,3.
 */
public class JeuMorpionModele {

    private int joueurCourant = 1; // C'est toujours le joueur 1 qui commence.

    /**
     * Lien du numéro du joueur vers son pion. On sacrifie une case du tableau
     * pour que ce soit simple.
     */
    private ContenuCase[] pions = {null, ContenuCase.PION1, ContenuCase.PION2};

    private PlateauMorpionModele plateauMorpion;

    public JeuMorpionModele(PlateauMorpionModele plateauMorpion) {
        this.plateauMorpion = plateauMorpion;
    }

    /**
     * Retourne le contenu de la case (i,j).
     * <p>
     * Si i et j sont hors limite, retourne 0 (comme pour une case vide).
     *
     * @param i
     * @param j
     * @return 1 (pour joueur 1), 2 (joueur 2) ou 0 (case vide).
     */
    public ContenuCase getPion(int i, int j) {
        return plateauMorpion.get(i, j);
    }

    /**
     * Joue dans la case i,j, si possible. Lève une exception si le coup n'est
     * pas possible.
     * <p>
     * Post condition : Après un succès: getPion(i,j) renverra le code du joueur
     * qui a joué dans cette case.
     * <p>
     * Post condition : le joueur courant sera l'autre joueur.
     * <p>
     * Post condition : .... (je vous laisse compléter, n'oubliez pas le MVC).
     *
     * @param i
     * @param j
     * @throws MorpionException si le coup n'est pas possible.
     */
    public void jouer(int i, int j) throws MorpionException {
        throw new RuntimeException("ECRIVEZ-MOI !!!");
    }

    /**
     * Retourne un enum qui décrit l'état du jeu.
     *
     * @return
     */
    public EtatJeu getEtatJeu() {
        EtatJeu etatJeu = EtatJeu.EN_COURS;

        List<List<ContenuCase>> alignements = new ArrayList<>();
        alignements.addAll(plateauMorpion.getLignes());
        alignements.addAll(plateauMorpion.getColonnes());
        alignements.addAll(plateauMorpion.getDiagonales());

        // On cherche si on a un alignement.
        for (List<ContenuCase> l : alignements) {
            boolean alignementTrouve = l.get(0) != ContenuCase.VIDE
                    && l.get(0) == l.get(1) && l.get(1) == l.get(2);
            if (alignementTrouve) {
                if (l.get(0) == ContenuCase.PION1) {
                    etatJeu = EtatJeu.JOUEUR1_GAGNE;
                } else {
                    etatJeu = EtatJeu.JOUEUR2_GAGNE;
                }
                break;
            }
        }

        if (etatJeu == EtatJeu.EN_COURS) {
            if (plateauMorpion.estPlein()) {
                etatJeu = EtatJeu.MATCH_NUL;
            }           
        }
        return etatJeu;
    }

    public int getJoueurCourant() {
        return joueurCourant;
    }
}
